// src/diff.ts
function diff(textLines, loadLines) {
  let i = 0;
  for (; i < textLines.length; i++) {
    if (textLines[i] !== loadLines[i]?.text) {
      break;
    }
    if (loadLines[i]?.loads.some(({ range }) => range.collapsed)) {
      break;
    }
  }
  let j = textLines.length - 1;
  let k = loadLines.length - 1;
  for (; j >= 0 && k >= 0; j--, k--) {
    if (textLines[j] !== loadLines[k]?.text) {
      break;
    }
    if (loadLines[k]?.loads.some(({ range }) => range.collapsed)) {
      break;
    }
  }
  if (i > j) {
    [i, j] = [j, i];
  }
  i = Math.max(0, i);
  j = Math.min(textLines.length - 1, j) + 1;
  if (textLines.length === loadLines.length && i + 1 === j) {
    return [i, j];
  }
  return [...expand(textLines, i, j)];
}
function* expand(textLines, start, end) {
  const side = matchTwoSides(Math.max(0, start + end - textLines.length));
  start -= side;
  end += side;
  function matchTwoSides(offset) {
    const pos = textLines.slice(0, start).indexOf(textLines[end], offset);
    if (pos === -1) {
      return 0;
    }
    for (let j = pos, k = end; j < start - 1; k++, j++) {
      if (textLines[j + 1] !== textLines[k + 1]) {
        return matchTwoSides(offset + 1);
      }
    }
    return start - pos;
  }
  const inner = new Set(textLines.slice(start, end));
  const left = [];
  for (let i = start - 1, it = end - 1; inner.has(textLines[i]); i--, it--) {
    const text = textLines[i];
    for (; it >= start; it--) {
      if (textLines[it] === text) {
        break;
      }
    }
    if (it >= start) {
      left.push(text);
    }
  }
  yield start - left.length;
  const right = [];
  for (let i = end, it = start; inner.has(textLines[i]); i++, it++) {
    const text = textLines[i];
    for (; it < end; it++) {
      if (textLines[it] === text) {
        break;
      }
    }
    if (it < end) {
      right.push(text);
    }
  }
  yield end + right.length;
}

// src/utils.ts
function debounce(func, {
  delay = 1500
} = {}) {
  let timer = void 0;
  return function(...args) {
    !timer && func.apply(this, args);
    timer = setTimeout(() => {
      timer = void 0;
    }, delay);
  };
}
function isArrayEqual(a, b) {
  if (a.length !== b.length) {
    return false;
  }
  for (let i = 0; i < a.length; i++) {
    if (a[i] !== b[i]) {
      return false;
    }
  }
  return true;
}

// src/index.ts
function createPlainShiki(shiki) {
  const isSupported = () => "CSS" in globalThis && "highlights" in CSS;
  function mount(el, options) {
    const {
      lang,
      themes = {
        light: "min-light",
        dark: "min-dark"
      },
      selector = (theme) => `.${theme}`,
      watch = true,
      delay = 0
    } = options;
    const debouncedUpdate = delay > 0 ? debounce(update, { delay }) : update;
    const stylesheet = new CSSStyleSheet();
    document.adoptedStyleSheets.push(stylesheet);
    const loadLines = [];
    const names = /* @__PURE__ */ new Set();
    if (isSupported()) {
      watch && el.addEventListener("input", debouncedUpdate);
      update();
    }
    function dispose() {
      watch && el.removeEventListener("input", debouncedUpdate);
      const idx = document.adoptedStyleSheets.indexOf(stylesheet);
      document.adoptedStyleSheets.splice(idx, 1);
      for (const name of names) {
        CSS.highlights.delete(name);
      }
    }
    function patch(loads, oldLoads) {
      for (const { range, name } of walkTokens(oldLoads)) {
        const highlight = CSS.highlights.get(name);
        if (!highlight) {
          continue;
        }
        highlight?.delete(range);
      }
      for (const { range, color, theme, name } of walkTokens(loads)) {
        const isDefault = theme === "light";
        let highlight = CSS.highlights.get(name);
        if (!highlight) {
          CSS.highlights.set(name, highlight = new Highlight());
          highlight.priority = isDefault ? 0 : 1;
        }
        if (!names.has(name)) {
          const rule = `${isDefault ? ":root" : selector(theme)}::highlight(${name}) { color: ${color}; }`;
          stylesheet.insertRule(rule);
          names.add(name);
        }
        highlight.add(range);
      }
    }
    function update() {
      const { innerText } = el;
      const textLines = innerText.split("\n");
      const textNodes = collectTextNodes(el);
      const [start, end] = diff(textLines, loadLines);
      const length = end - textLines.length + loadLines.length;
      const chunk = loadLines.splice(length);
      for (let i = start; i < length; i++) {
        patch([], loadLines[i]?.loads ?? []);
      }
      loadLines.length = end;
      loadLines.fill(null, start, end);
      loadLines.push(...chunk);
      let offset = textLines.slice(0, start).reduce((res, text) => res + text.length + 1, 0);
      const findNodeAndOffset = createFindNodeAndOffset(innerText, textNodes, offset);
      for (let i = start; i < textLines.length; i++) {
        const text = textLines[i];
        const tokenResult = shiki.codeToTokens(text, {
          lang,
          themes,
          cssVariablePrefix: "",
          defaultColor: false,
          grammarState: loadLines[i - 1]?.lastGrammarState
        });
        const loads = [];
        for (const token of tokenResult.tokens[0]) {
          const [startNode, startOffset] = findNodeAndOffset(offset + token.offset);
          const [endNode, endOffset] = findNodeAndOffset(offset + token.offset + token.content.length);
          const range = document.createRange();
          range.setStart(startNode, startOffset);
          range.setEnd(endNode, endOffset);
          loads.push({ token, range });
        }
        const loadLine = loadLines[i] ??= {};
        patch(loads, loadLine.loads ?? []);
        loadLine.loads = loads;
        loadLine.text = text;
        const oldScopes = loadLine.lastGrammarState?.getScopes() ?? [Number.NaN];
        const newScopes = tokenResult.grammarState?.getScopes() ?? [Number.NaN];
        const skip = isArrayEqual(oldScopes, newScopes);
        loadLine.lastGrammarState = tokenResult.grammarState;
        if (!skip) {
          offset += text.length + 1;
        } else break;
      }
    }
    return {
      dispose,
      update
    };
  }
  return {
    get isSupported() {
      return isSupported();
    },
    mount
  };
}
function* walkTokens(loads) {
  for (const { token, range } of loads) {
    if (typeof token.htmlStyle !== "object") {
      continue;
    }
    for (const theme in token.htmlStyle) {
      const color = token.htmlStyle[theme];
      const name = `shiki-${theme}-${color.slice(1).toLowerCase()}`;
      yield { range, color, theme, name };
    }
  }
}
function collectTextNodes(el) {
  const walker = document.createTreeWalker(el, NodeFilter.SHOW_TEXT);
  const textNodes = [];
  let node;
  while (node = walker.nextNode()) {
    textNodes.push(node);
  }
  return textNodes;
}
function createFindNodeAndOffset(innerText, textNodes, initialOffset) {
  let i = 0;
  let offset = 0;
  let isCorrect = false;
  if (initialOffset > 0) {
    find(initialOffset);
  }
  return find;
  function find(tokenOffset) {
    for (; i < textNodes.length; i++) {
      const node = textNodes[i];
      const { textContent } = node;
      if (!textContent) {
        continue;
      }
      if (!isCorrect) {
        offset = innerText.indexOf(textContent, offset);
        isCorrect = true;
      }
      if (offset + textContent.length < tokenOffset) {
        offset += textContent.length;
        isCorrect = false;
      } else break;
    }
    return [textNodes[i], tokenOffset - offset];
  }
}
export {
  createPlainShiki
};
